"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * This module must not import anything globally not workin in web-mode! if needed, require it within the functions
 *
 * NOTE, we Ignore the infrastructure-scripts libraries when bundling, so these can be used ...
 * We also put fs to empty! If you need other libs, add them to `node: { fs: empty }`
 */
const spa_component_1 = require("./spa-component");
const deepmerge = __importStar(require("deepmerge"));
const extract_domain_1 = __importDefault(require("extract-domain"));
/**
 * A Plugin to detect SinglePage-App-Components
 * @param props
 */
exports.SpaPlugin = (props) => {
    //console.log("configFilePath: " , props.configFilePath);
    const result = {
        // identify Isomorphic-App-Components
        applies: (component) => {
            return spa_component_1.isSinglePageApp(component);
        },
        // convert the component into configuration parts
        process: (component, childConfigs, infrastructureMode) => {
            const path = require('path');
            const webappBuildPath = path.join(require("../../../infrastructure-scripts/dist/infra-comp-utils/system-libs").currentAbsolutePath(), props.buildPath);
            const spaWebPack = require("../../../infrastructure-scripts/dist/infra-comp-utils/webpack-libs")
                .complementWebpackConfig(require("../../../infrastructure-scripts/dist/infra-comp-utils/webpack-libs")
                .createClientWebpackConfig("./" + path.join("node_modules", "infrastructure-components", "dist", "assets", "spa.js"), //entryPath: string,
            path.join(require("../../../infrastructure-scripts/dist/infra-comp-utils/system-libs").currentAbsolutePath(), props.buildPath), //use the buildpath from the parent plugin
            component.id, {
                __CONFIG_FILE_PATH__: require("../../../infrastructure-scripts/dist/infra-comp-utils/system-libs").pathToConfigFile(props.configFilePath),
            }, {}));
            // provide all client configs in a flat list
            const webpackConfigs = childConfigs.reduce((result, config) => result.concat(config.webpackConfigs), []);
            const copyAssetsPostBuild = () => {
                console.log("write the index.html!");
                require('fs').writeFileSync(path.join(webappBuildPath, component.stackName, "index.html"), `<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8" />
    <title>${component.stackName}</title>
  </head>
  <body>
    <noscript>You need to enable JavaScript to run this app.</noscript>
    <div id="root"></div>
    <script src="${component.stackName}.bundle.js"></script>
  </body>
</html>`);
            };
            const domain = childConfigs.map(config => config.domain).reduce((result, domain) => result !== undefined ? result : domain, undefined);
            const certArn = childConfigs.map(config => config.certArn).reduce((result, certArn) => result !== undefined ? result : certArn, undefined);
            const hostedZoneName = domain !== undefined ? extract_domain_1.default(domain.toString()) : {};
            const domainConfig = domain !== undefined && certArn !== undefined ? {
                // required of the SPA-domain-alias
                provider: {
                    customDomainName: domain,
                    hostedZoneName: hostedZoneName,
                    certArn: certArn
                },
            } : {};
            return {
                slsConfigs: deepmerge.all([
                    require("../../../infrastructure-scripts/dist/infra-comp-utils/sls-libs").toSpaSlsConfig(component.stackName, component.buildPath, component.region, domain),
                    domainConfig,
                    ...childConfigs.map(config => config.slsConfigs)
                ]),
                // add the server config 
                webpackConfigs: webpackConfigs.concat([spaWebPack]),
                postBuilds: childConfigs.reduce((result, config) => result.concat(config.postBuilds), [copyAssetsPostBuild]),
                environments: childConfigs.reduce((result, config) => (result !== undefined ? result : []).concat(config.environments !== undefined ? config.environments : []), []),
                stackName: component.stackName,
                assetsPath: undefined,
                buildPath: component.buildPath,
                region: component.region,
                domain: domain,
                certArn: certArn,
                // start the sls-stack offline does not work and does make sense either, we can use the hot-dev-mode
                supportOfflineStart: false,
                supportCreateDomain: false
            };
        }
    };
    return result;
};
//# sourceMappingURL=spa-plugin.js.map